@@ -64,6 +64,7 @@ int ipchains_ipac_deny(char *login);
 int ipchains_ipac_accept(char *login);
 int ipchains_ipac_check(void);
 int ipchains_ipac_remove_user(char *login);
+int ipchains_ipac_alarm();
 
 static const acc_agent_t interface_entry = {
 	"ipchains",
@@ -74,6 +75,7 @@ static const acc_agent_t interface_entry = {
 	ipchains_ipac_accept,
 	ipchains_ipac_check,
 	ipchains_ipac_remove_user,
+	ipchains_ipac_alarm,
 };
 
 const acc_agent_t *ipac_ag_interface_ipchains() {
@@ -209,7 +211,7 @@ struct runfile_line_type
 	if (frunfile == NULL) {
 		fprintf(stderr, "%s: cant open run file \"%s\": %s "
 				"(fetchipac -S not run?)\n",
-			me, RUNFILE, sys_errlist[errno]);
+			me, RUNFILE, strerror(errno));
 		return NULL;
 	}
 
@@ -237,7 +239,7 @@ struct runfile_line_type
 	if (!feof(frunfile))
 	{
 		fprintf(stderr, "%s: reading \"%s\": %s\n",
-			me, RUNFILE, sys_errlist[errno]);
+			me, RUNFILE, strerror(errno));
 		fclose(frunfile);
 		destroy_runfile_lines(result);
 		result = NULL;
@@ -314,7 +316,7 @@ read_ipchains(FILE *f, struct runfile_line_type *runfile,
 	lastrule = *firstrule = NULL;
 	for (nextline=runfile; nextline!=NULL; nextline=nextline->next)
 	{
-		cp = strchr(nextline->line, ' ');
+		cp = strchr(nextline->line, '|');
 		if (cp == 0)
 			continue;	/* bad entry */
 		rule = new_rule();
@@ -378,7 +380,7 @@ read_ipchains(FILE *f, struct runfile_line_type *runfile,
 					line_before=nextline,
 					nextline=nextline->next)
 				{
-					cp = strchr(nextline->line, ' ');
+					cp = strchr(nextline->line, '|');
 					if (cp == 0)
 						continue;	/* bad entry */
 					if (strncmp(nextline->line, chain_name,
@@ -535,7 +537,7 @@ setup_rules(void) {
 	frunfile = fopen(RUNFILE, "w");
 	if (!frunfile) {
 		fprintf(stderr, "%s: opening runfile \"%s\": %s\n",
-			me, RUNFILE, sys_errlist[errno]);
+			me, RUNFILE, strerror(errno));
 		return 1;
 	}
 	while(d)
@@ -602,8 +604,9 @@ setup_rules(void) {
 				printf("Creating chain \"%s\"\n", d->name+8);
 			ipfwc_create_chain(d->name+8);
 			strcpy(fw.label, d->name+8);
+			fprintf(frunfile, "%s|%%\n", chain);
 		} else
-			fprintf(frunfile,"%s %s\n", chain, d->name);
+			fprintf(frunfile,"%s|%s\n", chain, d->name);
 		if (verbose>2)
 			printf("Appending rule to chain \"%s\"\n", chain);
 		if (!ipfwc_append_entry(chain, &fw))
@@ -649,7 +652,7 @@ int ipchains_ipac_read(rule_type **firstrule)
 	if (fprocfile == NULL)
 	{
 		fprintf(stderr, "%s: cant open \"%s\": %s\n",
-				me, procfile, sys_errlist[errno]);
+				me, procfile, strerror(errno));
 		return 1;
 	}
 
@@ -767,3 +770,10 @@ ipchains_ipac_remove_user(char *login)
 {
 	return 0;
 }
+
+/* not implemented */
+int
+ipchains_ipac_alarm(void)
+{
+	return 1;
+}
