@@ -1,4 +1,4 @@
-/* Library which manipulates firewall rules.  Version $Revision: 1.33 $ */
+/* Library which manipulates firewall rules.  Version $Revision: 1.34 $ */
 
 /* Architecture of firewall rules is as follows:
  *
@@ -104,7 +104,7 @@ set_changed(TC_HANDLE_T h)
 	h->changed = 1;
 }
 
-#ifndef NDEBUG
+#ifdef IPTC_DEBUG
 static void do_check(TC_HANDLE_T h, unsigned int line);
 #define CHECK(h) do { if (!getenv("IPTC_NO_CHECK")) do_check((h), __LINE__); } while(0)
 #else
@@ -1268,11 +1268,11 @@ TC_ZERO_COUNTER(const IPT_CHAINLABEL chain,
 
 	e = index2entry(*handle, chainindex + rulenum);
 
-	if ((*handle)->counter_map[chainindex + rulenum].maptype
-			== COUNTER_MAP_NORMAL_MAP) {
+//	if ((*handle)->counter_map[chainindex + rulenum].maptype
+//			== COUNTER_MAP_NORMAL_MAP) {
 		(*handle)->counter_map[chainindex + rulenum].maptype
 			 = COUNTER_MAP_ZEROED;
-	}
+//	}
 
 	set_changed(*handle);
 
@@ -1696,7 +1696,7 @@ TC_COMMIT(TC_HANDLE_T *handle)
 				"counters alignment incorrect! Mail rusty!\n");
 			abort();
 		}
-		*kernptr = newcounters->counters;
+		*kernptr = &newcounters->counters;
 	}
 #endif /* KERNEL_64_USERSPACE_32 */
 
