@@ -4,7 +4,7 @@
  *
  * Fetch IP accounting stats
  * Copyright (C) 1997 - 2000 Moritz Both
- *           (C) 2001 - Al Zaharov   
+ *           (C) 2001 - 2002 Al Zaharov   
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -48,7 +48,9 @@
 
 int 		is_classic;
 
-int		verbose=0;
+int		dropzero = 0;
+
+int		verbose = 0;
 /** name of the program */
 const char 	*me;
 
@@ -61,7 +63,7 @@ const char	*datadir=ACCTDIR;
 char		*spoolfile;
 
 /** name of local host. */
-char 		hostname[256];
+char 		*hostname;
 
 /** name of config file. */
 char		conffile[256];
@@ -90,9 +92,22 @@ int		db_closed_by_timer;
 
 int in_transaction;
 
-int storage_opened;
+int storage_opened = 0;
 
-int access_opened;
+int access_opened = 1;
+
+int is_classic;
+char *storage_method_name = NULL;
+char *acc_agent_name = DEFAULT_AGENT;
+char *access_agent_name = DEFAULT_ACCESS;
+char *authhost = NULL;
+char *dbhost = NULL;
+char *dbname = NULL;
+char *dbport = NULL;
+char *dbuser = NULL;
+char *dbpass = NULL;
+int slogin = 0;
+char *ahost = NULL;
 
 static const char usage_message_first[] =
 "fetchipac Version " VERSION " - Usage:\n"
@@ -102,6 +117,11 @@ static const char usage_message_first[] =
 "-c|--classic [yes|no]\t simulate old-styled ipac\n"
 "-C|--config FILE\t specify config file to use\n"
 "-b|--batch\texecute commands which are fed via standard input\n"
+"-H|--hostname\thostname to work with\n"
+	"\t\tif not specified, fetchs with hostname() or hostname from config\n"
+	"\t\treports are generated for specified host\n\t\t\tor for all hosts,"
+	" if hostname is not specified\n"
+"-f|--filter\tsimple filter. show only *<filter expresion>* rules\n"
 "-d|--directory DIR\n"
 	"\t\tspecify data directory DIR\n"
 "-h|--help\tprint this help text\n"
@@ -150,6 +170,8 @@ try_to_login(char *login_name)
 	
 	if (tmp)
 		return 1;
+
+	while(lock(LOCKFILE));
 	
 	if (!(sf = fopen(STATUSFILE, "r"))) {
 		printf("Can't open status file %s\n", STATUSFILE);
@@ -182,6 +204,7 @@ try_to_login(char *login_name)
 		ul=ul->next;
 	}
 	fclose(sf);
+	unlock(LOCKFILE);
 	return tmp;
 }
 
@@ -198,6 +221,8 @@ try_to_logoff(char *login_name)
 	if (tmp)
 		return 1;
 	
+	while(lock(LOCKFILE));
+
 	if (!(sf = fopen(STATUSFILE, "r"))) {
 		printf("Can't open status file %s\n", STATUSFILE);
 		return 1;
@@ -229,6 +254,7 @@ try_to_logoff(char *login_name)
 		ul=ul->next;
 	}
 	fclose(sf);
+	unlock(LOCKFILE);
 	return tmp;
 }
 
@@ -243,7 +269,7 @@ get_user_list(void)
 		return 1;
 	}
 	while (u) {
-		printf("%s\t\t%s\n", u->login, u->ip);
+		printf("'%s'\t\t'%s'\n", u->login, u->ip);
 		u=u->next;
 	}
 	return 0;
@@ -259,10 +285,12 @@ list_raw(void)
 		access_agent->close();
 		return 1;
 	}
-	printf("Rule name | destination | interface | protocol | source"
-		    " | direction | policy\n");
+	printf("rule name            | destination   | iface | proto | "
+			"source              | destination         | direction | policy\n");
+	printf("------------------------------------------------------------------"
+			"------------------------------------------------------\n");
 	while(r) {
-		printf("%s | %s | %s | %s | %s %s | %s %s | %s | %s\n",r->name,
+		printf("%-20s | %-13s | %-5s | %-5s | %-18s %s | %-18s %s | %-9s | %-8s \n",r->name,
 		    r->dest,r->iface,r->protocol,r->snet,r->sport,r->dnet,
 		    r->dport,r->direction,r->policy);
 		r = r->next;
@@ -276,13 +304,13 @@ setup_access(int first)
 	rule_type *tmp;
 
 	if(acc_agent->init(0)) {
-		perror("Failed to initialize accounting agent, 
-						    aborting operations\n");
+		perror("Failed to initialize accounting agent, "
+						    "aborting operations\n");
 		return 1;
 	}
 	if(acc_agent->set(&tmp, first)) {
-		perror("Failed to setup accounting agent, 
-						    aborting operations\n");
+		perror("Failed to setup accounting agent, "
+						    "aborting operations\n");
 		return 1;
 	}
 	return 0;
@@ -307,12 +335,22 @@ setup_status()
 	return 0;
 }
 	
-int 
+int
 setup_agents(void)
 {
 	setup_access(1);
-	if (!is_classic)
+	if (!is_classic) {
 		setup_status();
+		if (slogin) {
+			user_list *ul;
+			if (access_agent->get_user_list(&ul)) return 1;
+			while(ul) {
+				if (strcmp(ul->login, "admin"))
+					try_to_login(ul->login);
+				ul=ul->next;
+			}
+		}
+	}
 	return 0;
 }
 
@@ -332,12 +370,17 @@ usage()
 /** set the hostname variable to the correct local host name. */
 static void set_hostname()
 {
-        if (gethostname(hostname, 256) != 0) {
-                strcpy(hostname, "(unknown host)");
-                fprintf(stderr, "%s warning: cant get hostname, "
+	char hname[256];
+	
+	if (!hostname) {
+		if (gethostname(hname, 256) != 0) {
+                	strcpy(hname, "(unknown host)");
+			fprintf(stderr, "%s warning: cant get hostname, "
                                 "using '%s': %s\n",
-                                me, hostname, sys_errlist[errno]);
-        }
+				me, hname, strerror(errno));
+		} else
+			hostname = xstrdup(hname);
+	}
 }
 
 /** create array storage_methods
@@ -425,8 +468,13 @@ check_cash(const data_record_type *data)
 	rule_type *d;
 	double old_cash, new_cash, price;
 	int pay_type;
+	int was_locked=0;
 	char *last_paid;
 
+	was_locked = lock(LOCKFILE);
+	if (was_locked) 
+		unlock(LOCKFILE);
+
 	d = data->firstrule;
 
 	while(d) {
@@ -454,22 +502,22 @@ check_cash(const data_record_type *data)
 						}
 						break;
 				}
-				if (old_cash!=new_cash) {
-    					access_agent->set_cash(d->name, 
-								    new_cash);
-					// drop connections to that user
-					if (new_cash<=0)
-						access_agent->logout(d->name, 
-								    new_cash);
+				if (old_cash!=new_cash)
+    					access_agent->set_cash(d->name, new_cash);
+				// drop connections to that user
+				if (new_cash<=0) {
+					access_agent->logout(d->name, new_cash);
 				}
 			}
 		}
 		d=d->next;
 	}
+	if (was_locked)
+		lock(LOCKFILE);
 	return 0;
 }
 
-int list_timestamps(timestamp_t tstart, timestamp_t tend)
+int list_timestamps(timestamp_t tstart, timestamp_t tend, char *ahost)
 {
 	timestamp_t *tlist, *t, before, after;
 	int n, i;
@@ -478,11 +526,11 @@ int list_timestamps(timestamp_t tstart, timestamp_t tend)
 	if (!storage_opened) {
 		storage_method->open(SM_OPEN_READONLY);
 		n = storage_method->list_timestamps(tstart, tend, &tlist,
-			&before, &after);
+			&before, &after, ahost);
 		storage_method->close();
 	} else
 		n = storage_method->list_timestamps(tstart, tend, &tlist,
-			&before, &after);
+			&before, &after, ahost);
 	
 	if (n > 0) {
 		printf(machine_output_format ? "%d\n" 
@@ -541,7 +589,8 @@ int list_timestamps(timestamp_t tstart, timestamp_t tend)
 */
 
 /* print a record on file descriptor f */
-int print_records(FILE *f, int n, const data_record_type *dr)
+int 
+print_records(FILE *f, int n, const data_record_type *dr)
 {
 	char buf[256];
 	rule_type *r;
@@ -553,8 +602,12 @@ int print_records(FILE *f, int n, const data_record_type *dr)
 	}
 	
 	for (i=0; i<n; i++, dr++) {
-        	fprintf(f, "%s%lu ", machine_output_format ? "ADD\n":"timstamp: ",
-			dr->timestamp);
+//		if (is_classic)
+        		fprintf(f, "%s%lu ", machine_output_format ? "ADD\n":"timstamp: ",
+				    dr->timestamp);
+//		else
+//			fprintf(f, "%s%lu ", machine_output_format ? "BILL\n":"timstamp: ",
+//				    dr->timestamp);
         	if (machine_output_format == 0) {
 		        nice_time(dr->timestamp, buf);
         		fprintf(f, "(%s) ", buf);
@@ -568,8 +621,8 @@ int print_records(FILE *f, int n, const data_record_type *dr)
 				dr->machine_name);
 		for (r = dr->firstrule; r != NULL; r=r->next)
 			fprintf(f, machine_output_format ?
-				"%Lu %Lu |%s|\n"
-				: "    bytes %14Lu pkts %12Lu %s\n",
+				"%llu %llu |%s|\n"
+				: "    bytes %14llu pkts %12llu %s\n",
 				r->bytes, r->pkts, r->name);
 		fprintf(f, "%s\n", machine_output_format ? ")" : "");
 		printf("\n");
@@ -578,7 +631,7 @@ int print_records(FILE *f, int n, const data_record_type *dr)
 }
 
 /* list record data of timestamps on standard input */
-int list_records()
+int list_records(char *filter)
 {
 	timestamp_t timestamp_b,timestamp_e;
 	data_record_type *dr;
@@ -620,7 +673,7 @@ int list_records()
 //			storage_method->open(SM_OPEN_READONLY);
 //			db_closed_by_timer = 0;
 //		}
-		n = storage_method->get_records(timestamp_b, timestamp_e, &dr);
+		n = storage_method->get_records(timestamp_b, timestamp_e, &dr, filter);
 //		if (!storage_opened)
 //			alarm(5);
 		print_records(stdout, n, dr);
@@ -646,7 +699,7 @@ int spool_record(const data_record_type *dr)
 	f = fopen(spoolfile, "a");
 	if (f == NULL) {
 		fprintf(stderr, "%s: cant open spool file \"%s\" - data lost: "
-				"%s\n", me, spoolfile, sys_errlist[errno]);
+				"%s\n", me, spoolfile, strerror(errno));
 		return 1;
 	}
 
@@ -656,7 +709,7 @@ int spool_record(const data_record_type *dr)
 	if (print_records(f, 1, dr) != 0) {
 		fprintf(stderr, "%s: cant write data record to spool file \""
 				"%s\", data lost: %s\n", me, spoolfile,
-				sys_errlist[errno]);
+				strerror(errno));
 		ret = 1;
 	}
 	fputs("\n", f);
@@ -682,7 +735,7 @@ void unspool()
 	f = fopen(spoolfile, "r+");
 	if (f == NULL) {
 		fprintf(stderr, "%s: cant open spool file \"%s\": %s\n",
-				me, spoolfile, sys_errlist[errno]);
+				me, spoolfile, strerror(errno));
 		return;
 	}
 
@@ -695,7 +748,7 @@ void unspool()
 			if (fseek(f, 0, SEEK_SET) != 0) {
 				fprintf(stderr, "%s: rewriting spool file: "
 						"cant seek: %s\n", me,
-						sys_errlist[errno]);
+						strerror(errno));
 				/* what now... just leave the spool file
 				 * untouched.
 				 */
@@ -703,14 +756,18 @@ void unspool()
 				return;
 			}
 			/* skip all records we dont want. */
-			while(num > 0 && fgets(line, 256, f) != NULL)
-			{
+			while(num > 0 && fgets(line, 256, f) != NULL) {
 				if (strncasecmp(line, "ADD", 3) == 0)
 					num--;
+				if (strncasecmp(line, "BILL", 4) == 0)
+					num--;
 			}
-			while(fgets(line, 256, f) != NULL)
+			while(fgets(line, 256, f) != NULL) {
 				if (strncasecmp(line, "ADD", 3) == 0)
 					break;
+				if (strncasecmp(line, "BILL", 4) == 0)
+					break;
+			}
 			/* we now have the first line we want to keep */
 			strcpy(spoolfile_new, spoolfile);
 			strcat(spoolfile_new, ".new");
@@ -719,17 +776,16 @@ void unspool()
 				fprintf(stderr, "%s: cant open new spool file "
 						"\"%s\": %s\n", me, 
 						spoolfile_new,
-						sys_errlist[errno]);
+						strerror(errno));
 				fclose(f);
 				return;
 			}
 			do {
-				if (fputs(line, newf) == EOF)
-				{
+				if (fputs(line, newf) == EOF) {
 					fprintf(stderr, "%s: cant write to file"
 						"\"%s\": %s\n", me, 
 						spoolfile_new,
-						sys_errlist[errno]);
+						strerror(errno));
 					fclose(newf);
 					fclose(f);
 					return;
@@ -738,7 +794,7 @@ void unspool()
 			if (ferror(f)) {
 				fprintf(stderr, "%s: error reading from spool "
 						"file: %s", me, 
-						sys_errlist[errno]);
+						strerror(errno));
 				fclose(f);
 				fclose(newf);
 				return;
@@ -748,7 +804,7 @@ void unspool()
 			if (ferror(newf)) {
 				fprintf(stderr, "%s: error writing to new spool"
 						" file: %s", me, 
-						sys_errlist[errno]);
+						strerror(errno));
 				fclose(newf);
 				return;
 			}
@@ -757,15 +813,14 @@ void unspool()
 				fprintf(stderr, "%s: error renaming \"%s\" to "
 						"\"%s\": %s\n", me, 
 						spoolfile_new, spoolfile,
-						sys_errlist[errno]);
+						strerror(errno));
 			}
 		}
-	}
-	else {
+	} else {
 		if (unlink(spoolfile) != 0)
 			fprintf(stderr, "%s: cant unlink \"%s\": %s - this "
 					"might result in corrupted data\n",
-					me, spoolfile, sys_errlist[errno]);
+					me, spoolfile, strerror(errno));
 	}
 }
 
@@ -788,9 +843,6 @@ int batchmode()
 int main(int argc, char **argv)
 {
 	rule_type *firstrule = NULL;
-	const char *storage_method_name = DEFAULT_STORAGE;
-	const char *acc_agent_name = DEFAULT_AGENT;
-	const char *access_agent_name = DEFAULT_ACCESS;
 	char *login_name = NULL;
 
 	void *node = NULL;
@@ -804,21 +856,26 @@ int main(int argc, char **argv)
 	const acc_agent_t **aap;
 	const access_agent_t **acp;
 	int record_stored;		/* 1 if a record could be stored */
+	char *filter=NULL;
+
+	FILE *configa;
 
 	static struct option long_options[] = {
 		{ "agent",	    required_argument,	NULL,	'a'	},
 		{ "login",	    required_argument,	NULL,	'L'	},
 		{ "logoff",	    required_argument,	NULL,	'O'	},
 		{ "config",	    required_argument,	NULL,	'C'	},
+		{ "filter",         required_argument,	NULL,	'f'	},
 		{ "start",          no_argument,	NULL,	'S'	},
 		{ "list-raw",       no_argument,	NULL,	'R'	},
 		{ "user-list",	    no_argument,	NULL,	'u'	},
+		{ "ahost",	    required_argument,  NULL,   'H'     },
 		{ "batch",	    no_argument,	NULL,	'b'	},
 		{ "directory",	    required_argument,	NULL,	'd'	},
 		{ "help",	    no_argument,	NULL,	'h'	},
 		{ "machine-output-format", no_argument,	NULL,	'm'	},
 		{ "records",	    optional_argument,	NULL,	'r'	},
-		{ "classic",	    optional_argument,	NULL,	'c'	},
+		{ "classic",	    required_argument,	NULL,	'c'	},
 		{ "storage-method", required_argument,	NULL,	's'	},
 		{ "timestamps",	    optional_argument,	NULL,	't'	},
 		{ "verbose",	    no_argument,	NULL,	'v'	},
@@ -828,11 +885,10 @@ int main(int argc, char **argv)
 	};
 
 	me = argv[0];
-	set_hostname();
 	mode = 0;	/* fetch data */
 	strncpy(conffile, CONFFILE, sizeof(conffile)-1); // default config
 
-	if (!memcmp(CLASSIC, "no", 2))
+	if (!memcmp(EMULATE_CLASSIC, "no", 2))
 		is_classic = 0;
 	else
 		is_classic = 1;
@@ -845,7 +901,7 @@ int main(int argc, char **argv)
 	/* parse command line */
 	while(1)
 	{
-		c = getopt_long(argc, argv, "a:L:O:C:SRubd:vhmr::c:s:t::e", 
+		c = getopt_long(argc, argv, "a:L:O:C:f:SRuH:bd:vhmr::c:s:t::e", 
 				long_options, NULL);
 		if (c==EOF)
 			break;
@@ -860,6 +916,13 @@ int main(int argc, char **argv)
 			case 'S':
 				mode = 5;
 				break;
+			case 'H':
+				hostname = xstrdup(optarg);
+				ahost = xstrdup(optarg);
+				break;
+			case 'f':
+				filter = xstrdup(optarg);
+				break;
 			case 'c':
 				if (memcmp(optarg, "yes", 3))
 					is_classic = 0;
@@ -924,6 +987,16 @@ int main(int argc, char **argv)
 		}
 	}
 
+	configa = fopen(conffile, "r");
+	if (configa != NULL) {
+		parse_config(configa);
+	} else {
+		fprintf(stderr, "error opening config file '%s': %s\n", conffile, strerror (errno));
+		return 2;
+	}
+	fclose(configa);
+	set_hostname();
+
 	/* find storage method */
 	for (smp = storage_methods; *smp != NULL; smp++)
 	{
@@ -951,7 +1024,7 @@ int main(int argc, char **argv)
 	}
 	acc_agent = *aap;
 
-	/* find accounting agent */
+	/* find access agent */
 	for (acp = access_agents; *acp != NULL; acp++) {
 		if (strcasecmp((*acp)->name, access_agent_name) == 0)
 			break;
@@ -963,7 +1036,16 @@ int main(int argc, char **argv)
 	}
 	access_agent = *acp;
 
-	access_agent->open(0);
+
+	if (access_agent->open(0)) {
+		fprintf(stderr, "%s: error while opening access database\n", me);
+		access_opened = 0;
+		fprintf(stderr, "cant operate while database access is absent\n");
+		acc_agent->alarm();
+		return 1;
+	}
+	spoolfile = xmalloc(strlen(datadir) + sizeof(SPOOLFILE) + 10);
+	sprintf(spoolfile, "%s/" SPOOLFILE, datadir);
 
 	/* the normal mode is 0. If another mode is set, we do what has
 	 * to be done here and leave main(). the normal fetch comes below
@@ -973,9 +1055,9 @@ int main(int argc, char **argv)
 	switch(mode)
 	{
 		case 1:
-			return list_timestamps(tstart, tend);
+			return list_timestamps(tstart, tend, ahost);
 		case 2:
-			return list_records();
+			return list_records(filter);
 		case 3:
 			return batchmode();
 		case 4: 
@@ -1016,9 +1098,6 @@ int main(int argc, char **argv)
 				"with fetchipac -S\n");
 		return 1;
 	}
-		
-	spoolfile = xmalloc(strlen(datadir) + sizeof(SPOOLFILE) + 10);
-	sprintf(spoolfile, "%s/" SPOOLFILE, datadir);
 
 	if (lock(LOCKFILE) == 0) {
 		struct stat *buf = NULL;
@@ -1038,24 +1117,48 @@ int main(int argc, char **argv)
 			}
 		free(buf);
 		time(&t);
+		// FIXME:
+		// Ugly hack to avoid some gdbm storage troubles :(
+		// Will be removed together with gdbm storage
+		// No, it will not be fixed soon %). only in 2.x :(
+/*
+		tmp = 1;
+		while (tmp>0) {
+			if (!storage_opened)
+				tmp = 0;// storage closed? nothing to do, just spool
+			else
+				tmp = storage_method->list_timestamps(t, t, &tlist,
+						&before, &after);
+			if (tmp>0) t++;
+		}
+*/		
 		if (acc_agent->read(&firstrule) == 0) {
 			data_record.timestamp = t;
 			data_record.firstrule = firstrule;
 			data_record.machine_name = hostname;
 			record_stored = 0;
 
-			if (storage_opened) {
-				if (storage_method->store_record(&data_record) 
-						== 0)
-					record_stored = 1;
-				else {
-					fprintf(stderr, "%s: storage reports e"
-							"rror on "
-							"store record\n", me);
+			if (dropzero) {
+				rule_type *rule1 = firstrule;
+				rule_type *rule2 = firstrule;
+				
+				while (rule1) {
+					if (rule1->pkts==0)
+						rule2->next=rule1->next;
+					else 
+						rule2=rule1;	
+					rule1=rule1->next;
 				}
-				if (!is_classic)
-					check_cash(&data_record);
 			}
+			
+			if (storage_opened) {
+				if (storage_method->store_record(&data_record) == 0)
+					record_stored = 1;
+				else
+					fprintf(stderr, "%s: storage reports error on store record\n", me);
+			}
+			if (!is_classic && access_opened)
+				check_cash(&data_record);
 
 			/* if our data could not be stored for some reason,
 			 * we put it in ASCII representation into a spool
@@ -1071,13 +1174,14 @@ int main(int argc, char **argv)
 			storage_method->close();
 		}
 
+		unlock(LOCKFILE);
 		/* reconfigure, if needed. supported only for neoclassic mode
 		 * we have to store data before reconfiguring
 		 * warning, there are posible traffic loss doing reconfigure
 		 * althought it is very small possibility. in case of data
 		 * blame me loss don't
 		 */
-		if ((mode==9) && (!is_classic)) {
+		if ((mode==9) && (!is_classic) && (access_opened)) {
 			user_list *ul=NULL, *ul1=NULL, *ulist=NULL;
 			FILE *sf;
 			char line[MAX_RULE_NAME_LENGTH];
@@ -1120,8 +1224,8 @@ int main(int argc, char **argv)
 				ul=ul->next;
 			}
 		}
-		unlock(LOCKFILE);
 	}
-	access_agent->close();
+	if (access_opened)
+		access_agent->close();
 	return 0;
 }
