@@ -134,7 +134,7 @@ static void print_gdbm_error(char *format, ...)
 	vfprintf(stderr, format, vl);
 	va_end(vl);
 	fprintf(stderr, ": %s / %s\n", gdbm_strerror(gdbm_errno),
-			sys_errlist[errno]);
+			strerror(errno));
 }
 
 /** called whenever corrupted data is detected. */
@@ -187,7 +187,7 @@ static GDBM_FILE ipac_open_gdbm(char *file, int read_write)
 	{
 		fprintf(stderr, "%s: cant open \"%s\": %s / %s\n",
 				me, file, gdbm_strerror(gdbm_errno),
-				sys_errlist[errno]);
+				strerror(errno));
 	}
 	return dbfile;
 }
@@ -324,7 +324,7 @@ static int add_timestamp(gdbm_timestamp *ts)
 	{
 		fprintf(stderr, "%s: cant fstat \"%s\" fd=%d: "
 				"%s\n", me, filename_timestamps,
-				fileno(ftimestamps), sys_errlist[errno]);
+				fileno(ftimestamps), strerror(errno));
 		return -1;
 	}
 
@@ -346,7 +346,7 @@ static int add_timestamp(gdbm_timestamp *ts)
 					" in "
 					"\"%s\": %s\n",
 					me, -offset, filename_timestamps,
-					sys_errlist[errno]);
+					strerror(errno));
 			return -1;
 		}
 		/* we remember all the timestamps we read in case we have
@@ -379,7 +379,7 @@ static int add_timestamp(gdbm_timestamp *ts)
 			fprintf(stderr, "%s: cant read item from "
 					"\"%s\": %s\n",
 					me, filename_timestamps,
-					ferror(ftimestamps) ? sys_errlist[errno]
+					ferror(ftimestamps) ? strerror(errno)
 					: "end of file");
 			return -1;
 		}
@@ -422,7 +422,7 @@ static int add_timestamp(gdbm_timestamp *ts)
 				" in "
 				"\"%s\": %s\n",
 				me, -offset, filename_timestamps,
-				sys_errlist[errno]);
+				strerror(errno));
 error_exit_with_free_tsold:
 		free(tsold_buffer);
 		return -1;
@@ -432,7 +432,7 @@ error_exit_with_free_tsold:
 		fprintf(stderr, "%s: cant write item to "
 				"\"%s\": %s\n",
 				me, filename_timestamps,
-				ferror(ftimestamps) ? sys_errlist[errno]
+				ferror(ftimestamps) ? strerror(errno)
 				: "end of file");
 		goto error_exit_with_free_tsold;
 	}
@@ -447,7 +447,7 @@ error_exit_with_free_tsold:
 			fprintf(stderr, "%s: cant write %d items to "
 					"\"%s\": %s\n",
 					me, n, filename_timestamps,
-					ferror(ftimestamps) ? sys_errlist[errno]
+					ferror(ftimestamps) ? strerror(errno)
 					: "end of file");
 			goto error_exit_with_free_tsold;
 		}
@@ -526,7 +526,7 @@ static int find_timestamp(timestamp_t timestamp, unsigned long *offset)
 	{
 		fprintf(stderr, "%s: cant fstat \"%s\" fd=%d: "
 				"%s\n", me, filename_timestamps,
-				fileno(ftimestamps), sys_errlist[errno]);
+				fileno(ftimestamps), strerror(errno));
 		return -1;
 	}
 
@@ -547,7 +547,7 @@ static int find_timestamp(timestamp_t timestamp, unsigned long *offset)
 		if (fread(&t1, sizeof(t1), 1, ftimestamps) != 1) {
 			fprintf(stderr, "%s: cant read from timestamps file:"
 					"%s\n", me, feof(ftimestamps) ?
-					"EOF" : sys_errlist[errno]);
+					"EOF" : strerror(errno));
 			return -1;
 		}
 // was '<=' i've changed it to '<' Kaiser
@@ -587,7 +587,7 @@ static int timestamp_step(unsigned long *offset, int backward,
 	if (fstat(fileno(ftimestamps), &statbuf) != 0)
 	{
 		fprintf(stderr, "%s: can stat timestamps file: %s\n", me,
-				sys_errlist[errno]);
+				strerror(errno));
 		return -1;
 	}
 
@@ -626,7 +626,7 @@ static int timestamp_step(unsigned long *offset, int backward,
 			{
 				fprintf(stderr, "%s: cant fseek on timestamps "
 						"file to offset %lu: %s\n", me, 
-						cache_start,sys_errlist[errno]);
+						cache_start,strerror(errno));
 				return -1;
 			}
 			if (fread(cache, sizeof(gdbm_timestamp), cachesize,
@@ -634,7 +634,7 @@ static int timestamp_step(unsigned long *offset, int backward,
 			{
 				fprintf(stderr, "%s: cant read from timestamps "
 						"file: %s\n", me,
-						sys_errlist[errno]);
+						strerror(errno));
 				return -1;
 			}
 		}
@@ -689,7 +689,7 @@ static int rebuild_timestamps()
 	if (fseek(ftimestamps, 0, SEEK_SET) != 0)
 	{
 		fprintf(stderr, "%s: cant seek on timestamps file: %s\n",
-				me, sys_errlist[errno]);
+				me, strerror(errno));
 		exit(1);
 	}
 	if (fwrite(array, sizeof(gdbm_timestamp), array_index, ftimestamps)
@@ -697,7 +697,7 @@ static int rebuild_timestamps()
 	{
 		fprintf(stderr, "%s: cant write %u elements to timestmaps "
 				"file: %s\n", me, array_index, 
-				sys_errlist[errno]);
+				strerror(errno));
 		exit(1);
 	}
 	fflush(ftimestamps);
@@ -759,7 +759,7 @@ open_error_out:
 							"estmaps file \"%s\""
 							": %s\n", me, 
 							filename_timestamps,
-							sys_errlist[errno]);
+							strerror(errno));
 					goto open_error_out;
 				}
 				close(fd);
@@ -767,7 +767,7 @@ open_error_out:
 		} else {
 			fprintf(stderr, "%s: cant stat timestmps file \"%s\":"
 					"%s\n", me, filename_timestamps, 
-					sys_errlist[errno]);
+					strerror(errno));
 			goto open_error_out;
 		}
 	}
@@ -778,7 +778,7 @@ open_error_out:
 		fprintf(stderr, "%s: cant open timestamps file \"%s\" for %s:"
 				" %s\n",
 				me, filename_timestamps, msg, 
-				sys_errlist[errno]);
+				strerror(errno));
 		goto open_error_out;
 	}
 
@@ -923,13 +923,19 @@ error_exit_with_free:
 /* list the timestamps. */
 int gdbm_ipac_list_timestamps(timestamp_t start, timestamp_t end, 
 		timestamp_t **data, timestamp_t *just_before,
-                        timestamp_t *just_after)
+                        timestamp_t *just_after, char *ahost)
 {
 	unsigned long ostart, oend;
 	int n, i;
 	gdbm_timestamp *p, *p1, gt;
 	timestamp_t *tsp;
 
+/*
+ * 	if (!ahost) {
+		fprintf(stderr, "Accounting hosts not supported by gdbm. Switch to postgre\n");
+		return -1;
+	}
+*/	
 	if (find_timestamp(start, &ostart) != 0)
 		return -1;
 	if (find_timestamp(end+1, &oend) != 0)
@@ -964,7 +970,7 @@ int gdbm_ipac_list_timestamps(timestamp_t start, timestamp_t end,
 	if (fseek(ftimestamps, ostart, SEEK_SET) != 0)
 	{
 		fprintf(stderr, "%s: cant fseek in timestamps file to offset "
-				"%lu: %s\n", me, ostart, sys_errlist[errno]);
+				"%lu: %s\n", me, ostart, strerror(errno));
 		free(p);
 		free(*data);
 		return -1;
@@ -973,7 +979,7 @@ int gdbm_ipac_list_timestamps(timestamp_t start, timestamp_t end,
 	{
 		fprintf(stderr, "%s: cant read %d items from timestamps file:"
 				" %s\n", me, n, feof(ftimestamps) ?
-				"EOF" : sys_errlist[errno]);
+				"EOF" : strerror(errno));
 		free(p);
 		free(*data);
 		return -1;
@@ -1115,7 +1121,7 @@ int gdbm_ipac_get_records(timestamp_t timestamp_s, timestamp_t timestamp_e,
 	        int n, i;
 
 	        n = gdbm_ipac_list_timestamps(timestamp_s-1, timestamp_e,
-								    &tlist,0,0);
+								    &tlist,0,0,NULL);
                 //FIXME: thou shall not delete accounting files from now until 
 						    //end of this function
                 *data = (data_record_type *)xmalloc(sizeof(data_record_type)*n);
